@@ -1,11 +1,47 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import sys, time, gc, threading
+import sys, time, gc, threading, MySQLdb
 from .. import es, imatrix
 from works import Works, WorksThread
 from subject import Subject, SubjectThread
 
+MaxThreadCount = 3
+
+#删除2个月半之前作品的推荐记录
+def deletePastWorksRecord():
+	vids = []
+	for hit in es.scrollSearch('works_video_read', 'video', {"query": {"range": {"publishtime": {"lt": getDayBeforeTime(75)}}}, "size":1000}):
+		vids.append(hit['_id'])
+		if len(vids) == 1000:
+			if not deletePastWorksRecordOnce(vids):
+				return False
+			vids = []
+	return deletePastWorksRecordOnce(vids)
+
+def deletePastWorksRecordOnce(vids):
+	return es.deleteStatisticsData('statistics_works_hot_fixed', {"query": {"terms": {"_routing": vids}}}, "recommendviewrecordsearch") and es.deleteStatisticsData('statistics_works_hot_fixed_backups', {"query": {"terms": {"_routing": vids}}}, "recommendviewrecordsearch")
+
+def getDauFactor():
+	zeroTime = getZeroTime() * 1000
+	startTime = long(zeroTime - 6 * 24 * 60 * 60 * 1000)
+	endTime = long(time.time() * 1000)
+
+	body = {"query": {"range": {"time": {"gte": startTime, "lte": endTime}}}, "size": 0, "aggs": {"dau": {"date_histogram": {"field": "time", "interval": "day", "time_zone":"+08:00", "min_doc_count": 0, "extended_bounds": {"min": startTime, "max": endTime}, "order": {"_key": "asc"}}, "aggs": {"count": {"cardinality": {"field": "uid"}}}}}}
+	buckets = es.searchStatisticsAggs('works_actions', body)['dau']['buckets']
+
+	#print(buckets)
+	rate = 0
+	for index in range(1, 5):
+		beforeCount = buckets[index - 1]['count']['value']
+		if beforeCount != 0:
+			rate += (buckets[index]['count']['value'] - beforeCount) * 1.0 / beforeCount
+
+	activeDiv = buckets[5]['count']['value'] * (1 + rate / 5) * 0.4
+	a = 1 / (5.5 * activeDiv) if activeDiv != 0 else 0
+	todayDau = buckets[6]['count']['value']
+	return min(10, max(1, 1 / (a * todayDau))) if todayDau != 0 else 1
+
 def getSubjectHistory():
 	subjectHistory = {}
 	for hit in es.searchStatisticsData('subject_history'):
@@ -20,36 +56,41 @@ def getWorksHistory():
 		worksHistory[history['vid']] = history
 	return worksHistory
 
-def getWorksList(aids=[], blackVids=[]):
+def getWorksList():
 	worksList = []
 	
-	for hit in es.scrollSearch('works_video_read', 'video', {"size":1000}):
-		works = toWorks(hit['_source'], aids, blackVids)
+	#取最近2个月的authorstate和state都是正常的作品
+	for hit in es.scrollSearch('works_video_read', 'video', {"query": {"bool": {"must": [{"term": {"state": {"value": 0}}}, {"term": {"authorstate": {"value": 0}}}, {"range": {"publishtime": {"gte": getDayBeforeTime(60)}}}]}}, "size":1000}):
+		works = toWorks(hit['_source'])
 		if works != None:
 			worksList.append(works)
 	return worksList
 
 def updateHistoryActions():
 	worksList = getWorksList()
-	subjectAids = []
 	subjectList = []
 
-	now = int(time.time())
+	endTime = getZeroTime()
 
 	worksThreads = []
 
 	while len(worksList) != 0:
-		if len(threading.enumerate()) > 20:
-			time.sleep(5)
+		if len(threading.enumerate()) > MaxThreadCount:
+			#time.sleep(1)
+			pass
 		else:
 			works = worksList.pop()
 			print(len(worksList), works.vid, works.aid)
-			t = WorksThread(works, now)
+			t = WorksThread(works, endTime)
 			worksThreads.append(t)
+			t.setDaemon(True)
 			t.start()
 
 	while len(threading.enumerate()) != 1:
-		time.sleep(5)
+		#time.sleep(1)
+		pass
+
+	subjectAids = []
 
 	for t in worksThreads:
 		works = t.works
@@ -69,17 +110,20 @@ def updateHistoryActions():
 	subjectThreads = []
 
 	while len(subjectList) != 0:
-		if len(threading.enumerate()) > 20:
-			time.sleep(5)
+		if len(threading.enumerate()) > MaxThreadCount:
+			#time.sleep(1)
+			pass
 		else:
 			subject = subjectList.pop()
 			print(len(subjectList), subject.aid)
-			t = SubjectThread(subject, now)
+			t = SubjectThread(subject, endTime)
 			subjectThreads.append(t)
+			t.setDaemon(True)
 			t.start()
 
 	while len(threading.enumerate()) != 1:
-		time.sleep(5)
+		#time.sleep(1)
+		pass
 
 	for t in subjectThreads:
 		subjectList.append(t.subject)
@@ -90,31 +134,28 @@ def updateHistoryActions():
 	subjectHistory = {}
 	for subject in subjectList:
 		subjectHistory[subject.aid] = subject.toSaveEsData()
-	es.updateStatisticsData('subject_history', subjectHistory)
+	if not es.coverStatisticsDataReal('subject_history', subjectHistory):
+		return False
 
 	hot.calWorksScore(worksList)
 	worksHistory = {}
 	for works in worksList:
 		worksHistory[works.vid] = works.toSaveEsData()
-	es.updateStatisticsData('works_history', worksHistory)
+	return es.coverStatisticsDataReal('works_history', worksHistory)
 
 			
+def toWorks(worksInfo):
+	if worksInfo.has_key('vid') or worksInfo.has_key('aid') or worksInfo.has_key('title'):
+		return Works(worksInfo['vid'], worksInfo['aid'], worksInfo['title'], worksInfo['publishtime'], worksInfo['duration'] if worksInfo.has_key('duration') else 0)
 
-def toWorks(worksInfo, aids=[], blackVids=[]):
-	if not worksInfo.has_key('vid') or not worksInfo.has_key('aid') or (worksInfo.has_key('state') and worksInfo['state'] == 1) or (worksInfo.has_key('authorstate') and worksInfo['authorstate'] == 1):
-		return
-	if not worksInfo.has_key('publishtime') or worksInfo['publishtime'] == None:
-		return
-	aid = worksInfo['aid']
-	vid = worksInfo['vid']
-	if (len(aids) != 0 and not aid in aids) or vid in blackVids:
-		return
-	publishTime = int(time.time() - es.esTimeToTime(worksInfo['publishtime'], True)) / 3600
-	#最多取最近2个月的
-	#print(publishTime)
-	if publishTime > 1440:
-		return
-	return Works(vid, aid, worksInfo['title'], worksInfo['publishtime'], worksInfo['duration'] if worksInfo.has_key('duration') else 0)
+#当天零点秒数
+def getZeroTime():
+	now = time.localtime(time.time())
+	return time.mktime((now.tm_year, now.tm_mon, now.tm_mday, 0, 0, 0, 0, 0, 0))
+
+#60天之前的毫秒数
+def getDayBeforeTime(day):
+	return (long(time.time()) - day * 24 * 60 * 60) * 1000
 
 def cycleDisposePersonas():
 	while True:
@@ -141,6 +182,7 @@ def disposePersonasOnce(worksDict):
 
 	userActions = {}
 	worksActions = {}
+	worksPlayers = {}
 
 	aIndex = 0
 	for index, hit in enumerate(hits):
@@ -155,18 +197,23 @@ def disposePersonasOnce(worksDict):
 		actions = source['body'].split('\n')
 
 		for action in actions:
-			items = action.split('\t')
-			if len(items) > 2 and items[2].isdigit():
-				_id = str(startTime) + '$' + str(aIndex)
-				_time = es.timeToEsLocalTime(float(items[0]))
-				addUserAction(userActions, _id, createTime, items[2], _time, uid, items)
-				addWorksAction(worksDict, worksActions, _id, createTime, items[2], _time, uid, items)
+			try:
+				items = action.split('\t')
+				if len(items) > 2 and items[2].isdigit():
+					_id = str(startTime) + '$' + str(aIndex)
+					_time = es.timeToEsLocalTime(float(items[0]))
+					addUserAction(userActions, _id, createTime, items[2], _time, uid, items)
+					addWorksAction(worksDict, worksActions, _id, createTime, items[2], _time, uid, items)
+					addWorksPlayer(worksPlayers, items[2], uid, items)
+			except Exception as error:
+				print('dispose personas action error: ' + str(error))
 			aIndex += 1
 
 		startTime = max(startTime, createTime)
 
 	es.updateStatisticsData('user_actions', userActions)
 	es.updateStatisticsData('works_actions', worksActions)
+	es.updateStatisticsData('works_players', worksPlayers)
 	writeEndTime(startTime)
 
 	return startTime
@@ -198,6 +245,13 @@ def readWorks():
 5: 打开老的订阅页
 6: 打开播放页
 7: 主动打开推荐页
+8: 进入首页(公众号)
+9: 进入首页(微信分享)
+10: 进入首页(海报扫码)
+11: 进入播放页(公众号)
+12: 进入播放页(微信分享)
+13: 进入播放页(海报扫码)
+14: 微信版本号
 '''
 
 def addUserAction(userActions, _id, createTime, actionId, _time, uid, items):
@@ -213,10 +267,17 @@ def addUserAction(userActions, _id, createTime, actionId, _time, uid, items):
 	elif actionId == '15':
 		action = 7
 		value = 1
+	elif actionId == '17' and items[1] in ['pages/common/index/index', 'pages/common/videoplay/videoplay'] and items[3] in ['1', '2', '3']:
+		action = (7 if items[1] == 'pages/common/index/index' else 10) + int(items[3])
+		value = 1
+	elif actionId == '12':
+		action = 14
+		versionValueString = items[4].replace('.', '')
+		value = int(versionValueString) if versionValueString.isdigit() else -1
 	else:
 		return
 	userActions[_id] = {'createTime': createTime, 'time': _time, 'uid': uid, 'action': action, 'value': value}
-	if actionId == '7' and items[1] == 'pages/common/videoplay/videoplay' and items[3] == '1' and items[5] == '4':
+	if actionId == '7' and items[1] == 'pages/common/videoplay/videoplay' and items[3] == '1' and len(items) >= 6 and items[5] == '4':
 		userActions[_id + '$1'] = {'createTime': createTime, 'time': _time, 'uid': uid, 'action': 3, 'value': 1}
 
 '''
@@ -241,6 +302,8 @@ def addUserAction(userActions, _id, createTime, actionId, _time, uid, items):
 showMap = {'3': 3, '20': 6, '0': 8, '1': 10}
 showPlayMap = {'4': 5, '1': 7, '2': 9, '3': 11}
 
+simpleStatisticsMap = {'4': 12, '10': 14, '11': 15, '2': 16, '3': 17}
+
 def addWorksAction(worksDict, worksActions, _id, createTime, actionId, _time, uid, items):
 	if actionId == '6':
 		vid = items[3]
@@ -250,10 +313,18 @@ def addWorksAction(worksDict, worksActions, _id, createTime, actionId, _time, ui
 		vid = items[3]
 		action = showMap[items[4]]
 		value = 1
-	elif actionId == '7' and items[1] == 'pages/common/videoplay/videoplay' and items[3] == '1' and showPlayMap.has_key(items[5]):
+	elif actionId == '7' and items[1] == 'pages/common/videoplay/videoplay' and items[3] == '1' and len(items) >= 6 and showPlayMap.has_key(items[5]):
 		vid = items[4]
 		action = showPlayMap[items[5]]
 		value = 1
+	elif actionId == '5' and len(items) >= 5 and items[3] == '0':
+		vid = items[4]
+		action = 13
+		value = 1
+	elif simpleStatisticsMap.has_key(actionId):
+		vid = items[4 if actionId in [10, 11] else 3]
+		action = simpleStatisticsMap[actionId]
+		value = 1
 	else:
 		return
 	if not worksDict.has_key(vid):
@@ -263,7 +334,9 @@ def addWorksAction(worksDict, worksActions, _id, createTime, actionId, _time, ui
 	if actionId == '6' and (value >= 60 or value >= 0.9 * worksInfo['duration']):
 		worksActions[_id + '$1'] = {'createTime': createTime, 'time': _time, 'vid': vid, 'aid': worksInfo['aid'], 'duration': worksInfo['duration'], 'action': 2, 'uid': uid, 'value': 1}
 
-
+def addWorksPlayer(worksPlayers, actionId, uid, items):
+	if actionId == '1':
+		worksPlayers[items[3] + '$' + str(uid)] = {'vid': items[3], 'uid': uid}
 
 
 
